import 'package:cliniqally_calendar/src/calendar_page_controller.dart';
import 'package:cliniqally_calendar/src/models/date.dart';
import 'package:cliniqally_calendar/src/widgets/calendar_widget.dart';
import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'widgets/round_corner_card.dart';

import 'utils.dart';

class Calendar extends StatefulWidget {
  final Color? activeColor;
  final Color? backgroundColor;
  final TextStyle? selectedStyle;
  final TextStyle? textStyleDays;
  final TextStyle? textStyleWeekDay;
  final TextStyle? titleStyle;
  final bool weekendOpacityEnable;
  final double weekendOpacity;
  final bool disable;
  final ValueChanged<Date?>? onSelected;
  final DateTime? initialDate;

  final Widget? previous;
  final Widget? next;

  final double? space;

  Function()? addOnTap;

  bool isCustomized;

  Widget? child;

  Calendar(
      {Key? key,
      this.titleStyle,
      this.previous,
      this.next,
      this.activeColor,
      this.backgroundColor,
      this.selectedStyle,
      this.textStyleDays,
      this.textStyleWeekDay,
      this.onSelected,
      this.space,
      this.weekendOpacityEnable = false,
      this.weekendOpacity = 0.48,
      this.disable = false,
      this.initialDate,
      required this.isCustomized,
      this.addOnTap,
      this.child})
      : assert(weekendOpacityEnable == true
            ? textStyleWeekDay!.color != null
            : true),
        assert(
            weekendOpacityEnable == true ? textStyleDays!.color != null : true),
        super(key: key);

  @override
  _CalendarState createState() => _CalendarState();
}

class _CalendarState extends State<Calendar>
    with SingleTickerProviderStateMixin {
  late CalendarPageController controller;
  PageController? pageController;

  @override
  void initState() {
    pageController = PageController(initialPage: 1);
    controller = CalendarPageController(
      pageController,
      widget.initialDate,
    );
    widget.onSelected!(controller.dataCollection.currentMonth);

    super.initState();
  }

  @override
  void dispose() {
    controller.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return RoundCornerCard(
      borderRadius: 12,
      width: double.infinity,
      shadow: true,
      bluRad: 1,
      spreadRad: 1,
      opacity: 0.1,
      height: 656.h,
      color: widget.backgroundColor,
      child: Column(
        mainAxisSize: MainAxisSize.min,
        children: [
          Padding(
            padding: EdgeInsets.only(bottom: 10.h),
            child: Container(
              height: 52.h,
              decoration: const BoxDecoration(
                  color: Color(0xff189AB4),
                  borderRadius:
                      BorderRadius.vertical(top: Radius.circular(10))),
              child: Row(
                mainAxisSize: MainAxisSize.max,
                crossAxisAlignment: CrossAxisAlignment.center,
                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                children: [
                  StreamBuilder(
                      stream: controller.update,
                      initialData: PageDirection.none,
                      builder: (context, snapshot) {
                        return Padding(
                          padding: EdgeInsets.only(left: 10.w),
                          child: Text(
                            "${controller.dataCollection.currentMonth!.month!.month}, ${controller.dataCollection.currentMonth!.year}",
                            style: TextStyle(
                                color: Colors.white,
                                fontWeight: FontWeight.w600,
                                fontSize: 18.sp),
                          ),
                        );
                      }),
                  Padding(
                    padding: EdgeInsets.only(right: 12.w),
                    child: Row(
                      mainAxisSize: MainAxisSize.min,
                      children: [
                        if (widget.previous != null)
                          GestureDetector(
                            child: widget.previous,
                            onTap: widget.disable == true
                                ? null
                                : () {
                                    pageController!.previousPage(
                                        duration:
                                            const Duration(milliseconds: 200),
                                        curve: Curves.linear);
                                  },
                          ),
                        SizedBox(
                          width: widget.space,
                        ),
                        if (widget.previous != null)
                          GestureDetector(
                            child: widget.next,
                            onTap: widget.disable == true
                                ? null
                                : () {
                                    pageController!.nextPage(
                                        duration:
                                            const Duration(milliseconds: 200),
                                        curve: Curves.linear);
                                  },
                          )
                      ],
                    ),
                  ),
                ],
              ),
            ),
          ),
          Table(
            children: [
              TableRow(
                  children: List.generate(
                      7,
                      (index) => Center(
                            child: Text(
                              index.week,
                              style: widget.textStyleWeekDay!.copyWith(
                                  color: widget.weekendOpacityEnable
                                      ? index == 0 || index == 6
                                          ? widget.textStyleWeekDay!.color!
                                              .withOpacity(
                                                  widget.weekendOpacity)
                                          : widget.textStyleWeekDay!.color
                                      : null),
                            ),
                          ))),
            ],
          ),
          SizedBox(
            height: 6.h,
          ),
          Expanded(
            child: StreamBuilder<PageDirection>(
                stream: controller.update.stream,
                builder: (context, snapshot) => PageView(
                      controller: pageController,
                      onPageChanged: controller.onChage,
                      children: [
                        CalendarWidget(
                          key:
                              Key(controller.dataCollection.previousMonth!.key),
                          date: controller.dataCollection.previousMonth,
                          onSelected:
                              widget.disable == true ? null : widget.onSelected,
                          backgroundColor: widget.backgroundColor,
                          activeColor: widget.activeColor,
                          selectedStyle: widget.selectedStyle,
                          textStyleDays: widget.textStyleDays,
                          textStyleWeekDay: widget.textStyleWeekDay,
                          titleStyle: widget.titleStyle,
                          pageController: pageController,
                          previous: widget.previous,
                          next: widget.next,
                          weekendOpacityEnable: widget.weekendOpacityEnable,
                          weekendOpacity: widget.weekendOpacity,
                        ),
                        CalendarWidget(
                          key: Key(controller.dataCollection.currentMonth!.key),
                          date: controller.dataCollection.currentMonth,
                          onSelected:
                              widget.disable == true ? null : widget.onSelected,
                          backgroundColor: widget.backgroundColor,
                          activeColor: widget.activeColor,
                          selectedStyle: widget.selectedStyle,
                          textStyleDays: widget.textStyleDays,
                          textStyleWeekDay: widget.textStyleWeekDay,
                          titleStyle: widget.titleStyle,
                          pageController: pageController,
                          previous: widget.previous,
                          next: widget.next,
                          weekendOpacityEnable: widget.weekendOpacityEnable,
                          weekendOpacity: widget.weekendOpacity,
                        ),
                        CalendarWidget(
                          key: Key(controller.dataCollection.nextMonth!.key),
                          date: controller.dataCollection.nextMonth,
                          onSelected:
                              widget.disable == true ? null : widget.onSelected,
                          backgroundColor: widget.backgroundColor,
                          activeColor: widget.activeColor,
                          selectedStyle: widget.selectedStyle,
                          textStyleDays: widget.textStyleDays,
                          textStyleWeekDay: widget.textStyleWeekDay,
                          titleStyle: widget.titleStyle,
                          pageController: pageController,
                          previous: widget.previous,
                          next: widget.next,
                          weekendOpacityEnable: widget.weekendOpacityEnable,
                          weekendOpacity: widget.weekendOpacity,
                        ),
                      ],
                    )),
          ),
          Row(
            mainAxisAlignment: MainAxisAlignment.end,
            children: [
              Padding(
                padding: EdgeInsets.symmetric(horizontal: 8.w, vertical: 8.h),
                child: GestureDetector(
                  onTap: widget.addOnTap,
                  child: Container(
                    width: 80.w,
                    height: 34.h,
                    decoration: BoxDecoration(
                      border: Border.all(
                        width: 1.w,
                        color: const Color(0xff0BC4F1),
                      ),
                      borderRadius: BorderRadius.circular(20),
                    ),
                    child: Row(
                      mainAxisAlignment: MainAxisAlignment.center,
                      children: [
                        Icon(
                          Icons.add,
                          size: 16.sp,
                          color: Color(0xff0BC4F1),
                        ),
                        Text(
                          'Add',
                          style: TextStyle(
                              fontSize: 16.sp,
                              fontWeight: FontWeight.w400,
                              color: const Color(0xff0BC4F1)),
                        )
                      ],
                    ),
                  ),
                ),
              ),
            ],
          )
        ],
      ),
    );
  }
}
